/' (c) 2022 cepstrum.co.jp           '/
/' This is not free software.        '/
'
' adaptive filter test program, 16bit, 8kHz, monoral
'
' suppress periodic tone signal by adaptive filter
'
'  x ---+------------------------------>(+)---+---> e
'       |                            +   ^    |
'       |                         ^      | -  |
'       |                         |      |    |
'       |    +-------+ zx      +-----+ y |    |
'       +--->| delay +----+--->| FIR +---+    |
'            +-------+    |    +-----+        |
'            z^(-100)     |       |           |
'                         |    +-----+        |
'                         +--->| LMS |<-------+
'                              +-----+
'
 
#include "portaudio.bi"
#include "fbblas.bi"

#define FS        8000
#define PA_BUFLEN 1

#define MU        0.000001
#define LMSLEN    500
#define DLYLEN    100
#define PWRBETA   0.9995
#define DA_GAIN   2.0

type User_Data_Type
  dummy as any ptr
end type

dim user_data as User_Data_Type

dim stream           as any ptr 
dim errcode          as integer<32>

dim now_time         as string
dim past_time        as string
dim past_frame_count as integer

dim inchar           as string
dim keycode          as integer
dim i                as integer
dim tmp              as single

dim shared as integer     frame_count = 0
dim shared as integer<64> firix = 0
dim shared as single      xbuf(LMSLEN*2) 
dim shared as single      w(LMSLEN)

'--------------------------------------------

function delay100z(x as single) as single
static zbuf(100) as single = {                       _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0   _
}
  static ix as integer = 0
  dim    y  as single

  y=zbuf(ix)
  zbuf(ix)=x
  ix=(ix+1) mod 100
  return y
end function

'==========================================================================

function callbackfunc cdecl(                    _
  pread       as integer<16> ptr,               _
  pwrite      as integer<16> ptr,               _
  rwcount     as uinteger<32>,                  _
  timeInfo    as PaStreamCallbackTimeInfo ptr,  _
  flag        as uinteger<32>,                  _
  puser_data  as User_Data_Type ptr             _
) as integer

  dim i_ad    as integer<16>
  dim i_da    as integer<16>
  dim f_ad    as single
  dim f_da    as single
  dim x       as single
  dim y       as single
  dim e       as single  
  static zx   as single = 0.0
  static xpwr as single = 100.0

  i_ad=*pread
  f_ad=i_ad/32768.0
  x=f_ad

  zx=delay100z(x)
  xpwr=PWRBETA*xpwr+(1.0-PWRBETA)*zx*zx
  y=fbblas_fir(x, xbuf(), w(), LMSLEN, firix)
  e=x-y
  'fbblas_lms(w(), 2.0*MU*e, xbuf(), LMSLEN, firix)
  fbblas_lms(w(), 2.0*MU*e/xpwr, xbuf(), LMSLEN, firix)

  f_da=DA_GAIN*e
  i_da=int(f_da*32768.0)   ' int() : floor function
  *pwrite=i_da

  frame_count=frame_count+1
   
  return 0

end function

'==========================================================================

for i=0 to LMSLEN-1
  xbuf(i)=0.0
  xbuf(i+LMSLEN)=0.0
  w(i)=0.0
next

errcode=Pa_Initialize()

errcode=Pa_OpenDefaultStream(  _ 
  @stream,                     _  ' PaStream ptr ptr        
  1,                           _  ' input channels          
  1,                           _  ' output channels       
  8,                           _  ' sample format (paInt16)  
  FS,                          _  ' Fs                   
  PA_BUFLEN,                   _  ' frames per buffer     
  @callbackfunc,               _  ' callback function  
  @user_data                   _  ' user data      
) 

errcode=Pa_StartStream(stream)

past_time=""
past_frame_count=0
do
  inchar=inkey$
  keycode=asc(inchar)
  if keycode=3 or keycode=27 then           ' if ctrl-C or ESC
    errcode=Pa_StopStream(stream)
    errcode=Pa_CloseStream(stream)
    Pa_Terminate()
    '-------------------------------
    open "out.txt" for output as #1
    for i=0 to LMSLEN-1
      print #1, i, w(i)
    next
    close #1
    '-------------------------------
    system
  end if

  now_time=time
  if now_time<>past_time then
    print "hit ctrl-C or ESC key to exit"; " "; time; " "; frame_count-past_frame_count; " "; 
    print using "##.##%"; 100.0*Pa_GetStreamCpuLoad(stream)                      
    past_frame_count=frame_count
  end if
  past_time=now_time
loop


