/' (c) 2022 cepstrum.co.jp           '/
/' This is not free software.        '/
'
' suppress periodic tone signal by adaptive filter (prediction adaptive filter)
'
'  x ---+------------------------------>(+)---+---> e
'       |                            +   ^    |
'       |                         ^      | -  |
'       |                         |      |    |
'       |    +-------+ zx      +-----+ y |    |
'       +--->| delay +----+--->| FIR +---+    |
'            +-------+    |    +-----+        |
'            z^(-100)     |       |           |
'                         |    +-----+        |
'                         +--->| LMS |<-------+
'                              +-----+
'

#define LMSLEN  1000
#define DLYLEN  100
#define MU      0.000002
#define PWRBETA 0.9995

dim w(LMSLEN)    as single
dim xbuf(LMSLEN) as single

dim x            as single
dim zx           as single
dim y            as single
dim e            as single
dim xpwr         as single

dim s            as string
dim i            as integer

'--------------------------------------------
' conventional FIR filter function

function fir(        _
  x      as single,  _
  xbuf() as single,  _
  coef() as single,  _
  length as integer  _
) as single

  dim ix as integer
  dim y  as single

  for ix=length-1 to 1 step -1 
    xbuf(ix)=xbuf(ix-1)
  next
  xbuf(0)=x
  y=0.0
  for ix=0 to length-1
    y=y+xbuf(ix)*coef(ix)
  next
  return y
end function

'--------------------------------------------
' conventional LMS algorithm function

sub lms(             _
  w()    as single,  _
  e      as single,  _
  xbuf() as single,  _
  length as integer  _
) 
  dim ix as integer

  for ix=0 to length-1
    w(ix)=w(ix)+e*xbuf(ix)
  next
end sub

'--------------------------------------------

function delay100z(x as single) as single
static zbuf(100) as single = {                       _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0   _
}
  static ix as integer = 0
  dim    y  as single

  y=zbuf(ix)
  zbuf(ix)=x
  ix=(ix+1) mod 100
  return y
end function

'--------------------------------------------

for i=0 to LMSLEN-1
  w(i)   =0.0
  xbuf(i)=0.0
next

xpwr=100.0
do
  input s
  if s="" then
    exit do
  end if

  'print "///"; s; "///"

  x=val(s)
  zx=delay100z(x)
  xpwr=PWRBETA*xpwr+(1.0-PWRBETA)*zx*zx
  y=fir(zx, xbuf(), w(), LMSLEN)
  e=x-y
  lms(w(), 2.0*MU*e/xpwr, xbuf(), LMSLEN)

  print e
loop






