/' (c) 2022 cepstrum.co.jp           '/
/' This is not free software.        '/
'
' suppress periodic tone signal by adaptive filter (prediction adaptive filter)
'
'  x ---+------------------------------>(+)---+---> e
'       |                            +   ^    |
'       |                         ^      | -  |
'       |                         |      |    |
'       |    +-------+ zx      +-----+ y |    |
'       +--->| delay +----+--->| FIR +---+    |
'            +-------+    |    +-----+        |
'            z^(-100)     |       |           |
'                         |    +-----+        |
'                         +--->| LMS |<-------+
'                              +-----+
'

#include once "fbblas.bi"

#define LMSLEN  1000
#define DLYLEN  100
#define MU      0.000002
#define PWRBETA 0.9995

dim w(LMSLEN)      as single
dim xbuf(LMSLEN*2) as single

dim x              as single
dim zx             as single
dim y              as single
dim e              as single
dim xpwr           as single

dim firix          as integer<64>

dim s              as string
dim i              as integer

'--------------------------------------------

function delay100z(x as single) as single
static zbuf(100) as single = {                       _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  _
  0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0   _
}
  static ix as integer = 0
  dim    y  as single

  y=zbuf(ix)
  zbuf(ix)=x
  ix=(ix+1) mod 100
  return y
end function

'--------------------------------------------

for i=0 to LMSLEN-1
  w(i)          =0.0
  xbuf(i)       =0.0
  xbuf(i+LMSLEN)=0.0
next

firix=0
xpwr=100.0
do
  input s
  if s="" then
    exit do
  end if

  'print "///"; s; "///"

  x=val(s)
  zx=delay100z(x)
  xpwr=PWRBETA*xpwr+(1.0-PWRBETA)*zx*zx
  y=fbblas_fir(zx, xbuf(), w(), LMSLEN, firix)
  e=x-y
  fbblas_lms(w(), 2.0*MU*e/xpwr, xbuf(), LMSLEN, firix)

  print e
loop






