// (c) 2022 cepstrum.co.jp           
// This is not free software.        
//
// suppress periodic tone signal by adaptive filter (prediction adaptive filter)
//
//  x ---+------------------------------>(+)---+---> e
//       |                            +   ^    |
//       |                         ^      | -  |
//       |                         |      |    |
//       |    +-------+ zx      +-----+ y |    |
//       +--->| delay +----+--->| FIR +---+    |
//            +-------+    |    +-----+        |
//            z^(-100)     |       |           |
//                         |    +-----+        |
//                         +--->| LMS |<-------+
//                              +-----+
//

clear;

ADPLEN =1000;
DLYLEN =100;
MU     =0.000002;
PWRBETA=0.9995;

indata=loadwave('male_and_tone1.wav');
//indata=loadwave('male_and_tone2.wav');

datlen=length(indata);
outata=zeros(1:datlen);

w=zeros(1:ADPLEN);
xvec=zeros(1:ADPLEN);
dlybuf=zeros(1:DLYLEN);

zx=0.0;
xpwr=100.0;
for i=1:datlen

  x=indata(i);

  zx=dlybuf(1);
  xpwr=PWRBETA*xpwr+(1.0-PWRBETA)*zx*zx;
  dlybuf=[x, dlybuf(1:DLYLEN-1)];
  xvec=[zx, xvec(1:ADPLEN-1)];
  y=xvec*w';                         // FIR filter
  e=x-y;
  w=w+2.0*MU*e*xvec/xpwr;            // Normalized LMS algorithm
  //if datlen/10<i then w=w+2.0*MU*e*xvec/xpwr; end 

  outdata(i)=e;

end

outdata=0.95*outdata/max(abs(outdata));
savewave('out.wav', outdata, 8000);

plot(w);





